@@ -1,7 +1,7 @@
 /*
  * radclient.c	General radius packet debug tool.
  *
- * Version:	$Id$
+ * Version:	$Id: 4e04d0c5062137e73a08017a1e92597627748b23 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id$")
+RCSID("$Id: 4e04d0c5062137e73a08017a1e92597627748b23 $")
 
 #include <freeradius-devel/libradius.h>
 #include <freeradius-devel/conf.h>
@@ -37,6 +37,9 @@ RCSID("$Id$")
 
 #include <assert.h>
 
+#include "smbdes.h"
+#include "mschap.h"
+
 static int success = 0;
 static int retries = 3;
 static float timeout = 5;
@@ -82,6 +85,11 @@ typedef struct radclient_t {
 static radclient_t *radclient_head = NULL;
 static radclient_t *radclient_tail = NULL;
 
+const char *radclient_version = "radclient version " RADIUSD_VERSION_STRING
+#ifdef RADIUSD_VERSION_COMMIT
+" (git #" RADIUSD_VERSION_COMMIT ")"
+#endif
+", built on " __DATE__ " at " __TIME__;
 
 static void NEVER_RETURNS usage(void)
 {
@@ -91,6 +99,8 @@ static void NEVER_RETURNS usage(void)
 	fprintf(stderr, "  -c count    Send each packet 'count' times.\n");
 	fprintf(stderr, "  -d raddb    Set dictionary directory.\n");
 	fprintf(stderr, "  -f file     Read packets from file, not stdin.\n");
+	fprintf(stderr, "  -F          Print the file name, packet number and reply code.\n");
+	fprintf(stderr, "  -h          Print usage help information.\n");
 	fprintf(stderr, "  -i id       Set request id to 'id'.  Values may be 0..255\n");
 	fprintf(stderr, "  -n num      Send N requests/s\n");
 	fprintf(stderr, "  -p num      Send 'num' packets from a file in parallel.\n");
@@ -140,6 +150,44 @@ static void radclient_free(radclient_t *radclient)
 	free(radclient);
 }
 
+static int mschapv1_encode(VALUE_PAIR **request, const char *password)
+{
+	unsigned int i;
+	VALUE_PAIR *challenge, *response;
+	uint8_t nthash[16];
+
+	challenge = paircreate(PW_MSCHAP_CHALLENGE, PW_TYPE_OCTETS);
+	if (!challenge) {
+		fprintf(stderr, "GOT IT %d!\n", __LINE__);
+		return 0;
+	}
+
+	pairadd(request, challenge);
+	challenge->length = 8;
+	for (i = 0; i < challenge->length; i++) {
+		challenge->vp_octets[i] = fr_rand();
+	}
+
+	response = paircreate(PW_MSCHAP_RESPONSE, PW_TYPE_OCTETS);
+	if (!response) {
+		fprintf(stderr, "GOT IT %d!\n", __LINE__);
+		return 0;
+	}
+
+	pairadd(request, response);
+	response->length = 50;
+	memset(response->vp_octets, 0, response->length);
+
+	response->vp_octets[1] = 0x01; /* NT hash */
+
+	mschap_ntpwdhash(nthash, password);
+
+	smbdes_mschap(nthash, challenge->vp_octets,
+		      response->vp_octets + 26);
+	return 1;
+}
+
+
 /*
  *	Initialize a radclient data structure and add it to
  *	the global linked list.
@@ -218,6 +266,10 @@ static int radclient_init(const char *filename)
 		} else if ((vp = pairfind(radclient->request->vps, PW_CHAP_PASSWORD)) != NULL) {
 			strlcpy(radclient->password, vp->vp_strvalue,
 				sizeof(radclient->password));
+
+		} else if ((vp = pairfind(radclient->request->vps, PW_MSCHAP_PASSWORD)) != NULL) {
+			strlcpy(radclient->password, vp->vp_strvalue,
+				sizeof(radclient->password));
 		} else {
 			radclient->password[0] = '\0';
 		}
@@ -362,7 +414,7 @@ static int filename_walk(void *context, void *data)
 	 *	Read request(s) from the file.
 	 */
 	if (!radclient_init(filename)) {
-		return 1;	/* stop walking */
+		return -1;	/* stop walking */
 	}
 
 	return 0;
@@ -546,6 +598,12 @@ static int send_one_packet(radclient_t *radclient)
 						vp->vp_octets,
 						radclient->request->id, vp);
 				vp->length = 17;
+
+			} else if ((vp = pairfind(radclient->request->vps, PW_MSCHAP_PASSWORD)) != NULL) {
+				mschapv1_encode(&radclient->request->vps,
+						radclient->password);
+			} else if (fr_debug_flag) {
+				printf("WARNING: No password in the request\n");
 			}
 		}
 
@@ -876,7 +934,7 @@ int main(int argc, char **argv)
 			timeout = atof(optarg);
 			break;
 		case 'v':
-			printf("radclient: $Id$ built on " __DATE__ " at " __TIME__ "\n");
+			printf("%s", radclient_version);
 			exit(0);
 			break;
 		case 'x':
