@@ -1,7 +1,7 @@
 /*
  * eap_tls.c
  *
- * Version:     $Id: 7b9a32f130ac78d67baf319fff9c70a3531703b8 $
+ * Version:     $Id: 5c47513e62724cb621aeb55f65e0f6083294aafe $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: 7b9a32f130ac78d67baf319fff9c70a3531703b8 $")
+RCSID("$Id: 5c47513e62724cb621aeb55f65e0f6083294aafe $")
 
 #include <freeradius-devel/autoconf.h>
 #include <assert.h>
@@ -162,6 +162,8 @@ int eaptls_success(EAP_HANDLER *handler, int peap_flag)
 
 		if (handler->certs) {
 			pairadd(&vps, paircopy(handler->certs));
+
+			pairadd(&request->packet->vps, paircopy(handler->certs));
 		}
 
 		if (vps) {
@@ -790,7 +792,7 @@ static eaptls_status_t eaptls_operation(eaptls_status_t status,
 	 */
 	if (!tls_handshake_recv(handler->request, tls_session)) {
 		DEBUG2("TLS receive handshake failed during operation");
-		eaptls_fail(handler, tls_session->peap_flag);
+		SSL_CTX_remove_session(tls_session->ctx, tls_session->ssl->session);
 		return EAPTLS_FAIL;
 	}
 
