@@ -1,7 +1,7 @@
 /*
  * rlm_eap_ttls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: e81843ad9cd0f69e4f2d997390a825575f8f4ed3 $
+ * Version:     $Id: 41e1473943f8f723835a0edba0ea2f4ea06629b7 $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include <freeradius-devel/ident.h>
-RCSID("$Id: e81843ad9cd0f69e4f2d997390a825575f8f4ed3 $")
+RCSID("$Id: 41e1473943f8f723835a0edba0ea2f4ea06629b7 $")
 
 #include "eap_ttls.h"
 
@@ -628,6 +628,15 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 
 		rcode = RLM_MODULE_OK;
 
+		/*
+		 *	Delete MPPE keys & encryption policy.  We don't
+		 *	want these here.
+		 */
+		pairdelete(&reply->vps, ((311 << 16) | 7));
+		pairdelete(&reply->vps, ((311 << 16) | 8));
+		pairdelete(&reply->vps, ((311 << 16) | 16));
+		pairdelete(&reply->vps, ((311 << 16) | 17));
+
 		/*
 		 *	MS-CHAP2-Success means that we do NOT return
 		 *	an Access-Accept, but instead tunnel that
@@ -643,15 +652,6 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 			rcode = RLM_MODULE_HANDLED;
 			t->authenticated = TRUE;
 
-			/*
-			 *	Delete MPPE keys & encryption policy.  We don't
-			 *	want these here.
-			 */
-			pairdelete(&reply->vps, ((311 << 16) | 7));
-			pairdelete(&reply->vps, ((311 << 16) | 8));
-			pairdelete(&reply->vps, ((311 << 16) | 16));
-			pairdelete(&reply->vps, ((311 << 16) | 17));
-
 			/*
 			 *	Use the tunneled reply, but not now.
 			 */
@@ -663,7 +663,7 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 		} else { /* no MS-CHAP2-Success */
 			/*
 			 *	Can only have EAP-Message if there's
-			 *	no MS-CHAP2-Success.  (FIXME: EAP-MSCHAP?)
+			 *	no MS-CHAP2-Success.
 			 *
 			 *	We also do NOT tunnel the EAP-Success
 			 *	attribute back to the client, as the client
