@@ -1,7 +1,7 @@
 /*
  * rlm_files.c	authorization: Find a user in the "users" file.
  *
- * Version:	$Id: rlm_files.c,v 1.85 2007/11/23 13:46:59 aland Exp $
+ * Version:	$Id: ad8693f877a19189d607f862d2221a939e441b2e $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  */
 
 #include	<freeradius-devel/ident.h>
-RCSID("$Id: rlm_files.c,v 1.85 2007/11/23 13:46:59 aland Exp $")
+RCSID("$Id: ad8693f877a19189d607f862d2221a939e441b2e $")
 
 #include	<freeradius-devel/radiusd.h>
 #include	<freeradius-devel/modules.h>
@@ -464,7 +464,7 @@ static int file_common(struct file_instance *inst, REQUEST *request,
 		}
 
 		if (paircompare(request, request_pairs, pl->check, reply_pairs) == 0) {
-			DEBUG2("    %s: Matched entry %s at line %d",
+			RDEBUG2("%s: Matched entry %s at line %d",
 			       filename, match, pl->lineno);
 			found = 1;
 			check_tmp = paircopy(pl->check);
